package xyz.lychee.lagfixer.libs.adventure.text.event;

import java.net.URL;
import java.util.Objects;
import java.util.function.Consumer;
import java.util.stream.Stream;
import xyz.lychee.lagfixer.libs.adventure.audience.Audience;
import xyz.lychee.lagfixer.libs.adventure.builder.AbstractBuilder;
import xyz.lychee.lagfixer.libs.adventure.internal.Internals;
import xyz.lychee.lagfixer.libs.adventure.text.format.Style;
import xyz.lychee.lagfixer.libs.adventure.text.format.StyleBuilderApplicable;
import xyz.lychee.lagfixer.libs.adventure.util.Index;
import xyz.lychee.lagfixer.libs.annotations.NotNull;
import xyz.lychee.lagfixer.libs.annotations.Nullable;
import xyz.lychee.lagfixer.libs.examination.Examinable;
import xyz.lychee.lagfixer.libs.examination.ExaminableProperty;

public final class ClickEvent implements Examinable, StyleBuilderApplicable {
   private final Action action;
   private final String value;

   public static @NotNull ClickEvent openUrl(final @NotNull String url) {
      return new ClickEvent(ClickEvent.Action.OPEN_URL, url);
   }

   public static @NotNull ClickEvent openUrl(final @NotNull URL url) {
      return openUrl(url.toExternalForm());
   }

   public static @NotNull ClickEvent openFile(final @NotNull String file) {
      return new ClickEvent(ClickEvent.Action.OPEN_FILE, file);
   }

   public static @NotNull ClickEvent runCommand(final @NotNull String command) {
      return new ClickEvent(ClickEvent.Action.RUN_COMMAND, command);
   }

   public static @NotNull ClickEvent suggestCommand(final @NotNull String command) {
      return new ClickEvent(ClickEvent.Action.SUGGEST_COMMAND, command);
   }

   public static @NotNull ClickEvent changePage(final @NotNull String page) {
      return new ClickEvent(ClickEvent.Action.CHANGE_PAGE, page);
   }

   public static @NotNull ClickEvent changePage(final int page) {
      return changePage(String.valueOf(page));
   }

   public static @NotNull ClickEvent copyToClipboard(final @NotNull String text) {
      return new ClickEvent(ClickEvent.Action.COPY_TO_CLIPBOARD, text);
   }

   public static @NotNull ClickEvent callback(final @NotNull ClickCallback<Audience> function) {
      return ClickCallbackInternals.PROVIDER.create((ClickCallback)Objects.requireNonNull(function, "function"), ClickCallbackOptionsImpl.DEFAULT);
   }

   public static @NotNull ClickEvent callback(final @NotNull ClickCallback<Audience> function, final ClickCallback.@NotNull Options options) {
      return ClickCallbackInternals.PROVIDER.create((ClickCallback)Objects.requireNonNull(function, "function"), (ClickCallback.Options)Objects.requireNonNull(options, "options"));
   }

   public static @NotNull ClickEvent callback(final @NotNull ClickCallback<Audience> function, final @NotNull Consumer<@NotNull ClickCallback.Options.Builder> optionsBuilder) {
      return ClickCallbackInternals.PROVIDER.create((ClickCallback)Objects.requireNonNull(function, "function"), (ClickCallback.Options)AbstractBuilder.configureAndBuild(ClickCallback.Options.builder(), (Consumer)Objects.requireNonNull(optionsBuilder, "optionsBuilder")));
   }

   public static @NotNull ClickEvent clickEvent(@NotNull final @NotNull Action action, final @NotNull String value) {
      return new ClickEvent(action, value);
   }

   private ClickEvent(@NotNull final @NotNull Action action, final @NotNull String value) {
      this.action = (Action)Objects.requireNonNull(action, "action");
      this.value = (String)Objects.requireNonNull(value, "value");
   }

   @NotNull
   public @NotNull Action action() {
      return this.action;
   }

   public @NotNull String value() {
      return this.value;
   }

   public void styleApply(final Style.@NotNull Builder style) {
      style.clickEvent(this);
   }

   public boolean equals(final @Nullable Object other) {
      if (this == other) {
         return true;
      } else if (other != null && this.getClass() == other.getClass()) {
         ClickEvent that = (ClickEvent)other;
         return this.action == that.action && Objects.equals(this.value, that.value);
      } else {
         return false;
      }
   }

   public int hashCode() {
      int result = this.action.hashCode();
      result = 31 * result + this.value.hashCode();
      return result;
   }

   public @NotNull Stream<? extends ExaminableProperty> examinableProperties() {
      return Stream.of(ExaminableProperty.of("action", this.action), ExaminableProperty.of("value", this.value));
   }

   public String toString() {
      return Internals.toString(this);
   }

   public static enum Action {
      OPEN_URL("open_url", true),
      OPEN_FILE("open_file", false),
      RUN_COMMAND("run_command", true),
      SUGGEST_COMMAND("suggest_command", true),
      CHANGE_PAGE("change_page", true),
      COPY_TO_CLIPBOARD("copy_to_clipboard", true);

      public static final Index<String, Action> NAMES = Index.create(Action.class, (constant) -> {
         return constant.name;
      });
      private final String name;
      private final boolean readable;

      private Action(final String name, final boolean readable) {
         this.name = name;
         this.readable = readable;
      }

      public boolean readable() {
         return this.readable;
      }

      public @NotNull String toString() {
         return this.name;
      }
   }
}
